@@ -5,10 +5,12 @@ import math
 import numpy as np
 
 import qlazy.config as cfg
+from qlazy.util import is_unitary_gate
 from qlazy.QState import QState
-from qlazy.lib.densop_mcx import densop_mcx
+from qlazy.QObject import QObject
+from qlazy.lib.densop_func import densop_operate_qcirc
 
-class DensOp(ctypes.Structure):
+class DensOp(ctypes.Structure, QObject):
     """ Density Operator
 
     Attributes
@@ -82,28 +84,6 @@ class DensOp(ctypes.Structure):
 
         return str(self.get_elm())
 
-    def reset(self, qid=None):
-        """
-        reset to |0><0| state.
-
-        Parameters
-        ----------
-        qid : list
-            qubit id's list to reset.
-
-        Returns
-        -------
-        densop : instance of DensOp
-            reset density operator.
-
-        Notes
-        -----
-        If 'qid' is not set, whole system is reset.
-
-        """
-        densop_reset(self, qid=qid)
-        return self
-
     @classmethod
     def mix(cls, densop=None, prob=None):
         """
@@ -1116,7 +1096,8 @@ class DensOp(ctypes.Structure):
         eigvals_A, eigvecs_A = DensOp.mat_spectrum(self.get_elm())
         eigvals_B, eigvecs_B = DensOp.mat_spectrum(densop.get_elm())
 
-        P = np.dot(np.conjugate(eigvecs_A.T), eigvecs_B)
+        # P = np.dot(np.conjugate(eigvecs_A.T), eigvecs_B)
+        P = np.dot(np.conjugate(eigvecs_A), eigvecs_B.T)
         P = np.conjugate(P)*P
 
         diag_A = [eigvals_A[i]*np.log2(eigvals_A[i]) for i in
@@ -1133,737 +1114,68 @@ class DensOp(ctypes.Structure):
         relent = relent_A - relent_B
         return relent
 
-    # 1-qubit gate
+    # operate gate
 
-    def x(self, q0):
+    def operate_gate(self, kind=None, qid=None, phase=0.0, **kwargs):
         """
-        operate X gate.
+        operate gate
 
         Parameters
         ----------
-        q0 : int
-            qubit id.
-
-        Returns
-        -------
-        self : instance of DensOp
-
-        """
-        densop_operate_qgate(self, kind=cfg.PAULI_X, phase=cfg.DEF_PHASE, qid=[q0])
-        return self
-
-    def y(self, q0):
-        """
-        operate Y gate.
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id.
-
-        Returns
-        -------
-        self : instance of DensOp
-
-        """
-        densop_operate_qgate(self, kind=cfg.PAULI_Y, phase=cfg.DEF_PHASE, qid=[q0])
-        return self
-
-    def z(self, q0):
-        """
-        operate Z gate.
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id.
-
-        Returns
-        -------
-        self : instance of DensOp
-
-        """
-        densop_operate_qgate(self, kind=cfg.PAULI_Z, phase=cfg.DEF_PHASE, qid=[q0])
-        return self
-
-    def xr(self, q0):
-        """
-        operate root X gate.
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id.
-
-        Returns
-        -------
-        self : instance of DensOp
-
-        """
-        densop_operate_qgate(self, kind=cfg.ROOT_PAULI_X, phase=cfg.DEF_PHASE, qid=[q0])
-        return self
-
-    def xr_dg(self, q0):
-        """
-        operate root X dagger gate
-        (hermmitian conjugate of root X gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id.
-
-        Returns
-        -------
-        self : instance of DensOp
-
-        """
-        densop_operate_qgate(self, kind=cfg.ROOT_PAULI_X_, phase=cfg.DEF_PHASE, qid=[q0])
-        return self
-
-    def h(self, q0):
-        """
-        operate H gate (hadamard gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id.
-
-        Returns
-        -------
-        self : instance of DensOp
-
-        """
-        densop_operate_qgate(self, kind=cfg.HADAMARD, phase=cfg.DEF_PHASE, qid=[q0])
-        return self
-
-    def s(self, q0):
-        """
-        operate S gate.
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id.
-
-        Returns
-        -------
-        self : instance of DensOp
-
-        """
-        densop_operate_qgate(self, kind=cfg.PHASE_SHIFT_S, phase=cfg.DEF_PHASE, qid=[q0])
-        return self
-
-    def s_dg(self, q0):
-        """
-        operate S dagger gate (hermitian conjugate of S gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id.
-
-        Returns
-        -------
-        self : instance of DensOp
-
-        """
-        densop_operate_qgate(self, kind=cfg.PHASE_SHIFT_S_, phase=cfg.DEF_PHASE, qid=[q0])
-        return self
-
-    def t(self, q0):
-        """
-        operate T gate.
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id.
-
-        Returns
-        -------
-        self : instance of DensOp
-
-        """
-        densop_operate_qgate(self, kind=cfg.PHASE_SHIFT_T, phase=cfg.DEF_PHASE, qid=[q0])
-        return self
-
-    def t_dg(self, q0):
-        """
-        operate T dagger gate (hermitian conjugate of T gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id.
-
-        Returns
-        -------
-        self : instance of DensOp
-
-        """
-        densop_operate_qgate(self, kind=cfg.PHASE_SHIFT_T_, phase=cfg.DEF_PHASE, qid=[q0])
-        return self
-
-    def rx(self, q0, phase=cfg.DEF_PHASE):
-        """
-        operate RX gate (rotation around X-axis).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id.
+        kind : int
+            kind of the gate
+        qid : list
+            quantum id list
         phase : float
-            rotation angle (unit of angle is PI radian).
+            phase for rotation gate
 
         Returns
         -------
         self : instance of DensOp
+            quantum state
 
         """
-        densop_operate_qgate(self, kind=cfg.ROTATION_X, phase=phase, qid=[q0])
+        if kind == cfg.RESET:
+            densop_reset(self, qid=qid)
+        elif is_unitary_gate(kind):
+            densop_operate_qgate(self, kind=kind, qid=qid, phase=phase)
+        else:
+            raise ValueError("gate: {} is not supported.".format(cfg.GATE_STRING[kind]))
         return self
 
-    def ry(self, q0, phase=cfg.DEF_PHASE):
+    # operate qcirc
+
+    def operate_qcirc(self, qcirc, qctrl=None):
         """
-        operate RY gate (rotation around Y-axis).
+        operate quantum circuit
 
         Parameters
         ----------
-        q0 : int
-            qubit id.
-        phase : float
-            rotation angle (unit of angle is PI radian).
-
-        Returns
-        -------
-        self : instance of DensOp
-
-        """
-        densop_operate_qgate(self, kind=cfg.ROTATION_Y, phase=phase, qid=[q0])
-        return self
-
-    def rz(self, q0, phase=cfg.DEF_PHASE):
-        """
-        operate RZ gate (rotation around Z-axis).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id.
-        phase : float
-            rotation angle (unit of angle is PI radian).
-
-        Returns
-        -------
-        self : instance of DensOp
-
-        """
-        densop_operate_qgate(self, kind=cfg.ROTATION_Z, phase=phase, qid=[q0])
-        return self
-
-    def p(self, q0, phase=cfg.DEF_PHASE):
-        """
-        operate P gate (phase shift gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id.
-        phase : float
-            rotation angle (unit of angle is PI radian).
+        qcirc : instance of QCirc
+            quantum circuit
+        qctrl : int
+            control qubit id
 
         Returns
         -------
         self : instance of DensOp
+            quantum state after executing the quantum circuit
 
         Notes
         -----
-        matrix expression is following...
-        | 1.0 0.0             |
-        | 0.0 exp(i*phase*PI) |
+        The quantum circut must be unintary.
 
         """
-        densop_operate_qgate(self, kind=cfg.PHASE_SHIFT, phase=phase, qid=[q0])
-        return self
+        if qcirc.is_unitary() is False:
+            raise ValueError("qcirc must be clifford quantum circuit.")
+        if self.qubit_num < qcirc.qubit_num:
+            raise ValueError("qubit number of quantum state must be equal or larger than the quantum circuit size.")
 
-    # 2-qubit gate
-
-    def cx(self, q0, q1):
-        """
-        operate CX gate (controlled X gate, controlled NOT gate, CNOT gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id (control qubit).
-        q1 : int
-            qubit id (target qubit).
-
-        Returns
-        -------
-        self : instance of DensOp
-
-        """
-        densop_operate_qgate(self, kind=cfg.CONTROLLED_X, phase=cfg.DEF_PHASE, qid=[q0, q1])
-        return self
-
-    def cy(self, q0, q1):
-        """
-        operate CY gate (controlled X gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id (control qubit).
-        q1 : int
-            qubit id (target qubit).
-
-        Returns
-        -------
-        self : instance of DensOp
-
-        """
-        densop_operate_qgate(self, kind=cfg.CONTROLLED_Y, phase=cfg.DEF_PHASE, qid=[q0, q1])
-        return self
-
-    def cz(self, q0, q1):
-        """
-        operate CZ gate (controlled Z gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id (control qubit).
-        q1 : int
-            qubit id (target qubit).
-
-        Returns
-        -------
-        self : instance of DensOp
-
-        """
-        densop_operate_qgate(self, kind=cfg.CONTROLLED_Z, phase=cfg.DEF_PHASE, qid=[q0, q1])
-        return self
-
-    def cxr(self, q0, q1):
-        """
-        operate CXR gate (controlled root X gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id (control qubit).
-        q1 : int
-            qubit id (target qubit).
-
-        Returns
-        -------
-        self : instance of DensOp
-
-        """
-        densop_operate_qgate(self, kind=cfg.CONTROLLED_XR, phase=cfg.DEF_PHASE, qid=[q0, q1])
-        return self
-
-    def cxr_dg(self, q0, q1):
-        """
-        operate CXR dagger gate (controlled XR dagger gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id (control qubit).
-        q1 : int
-            qubit id (target qubit).
-
-        Returns
-        -------
-        self : instance of DensOp
-
-        """
-        densop_operate_qgate(self, kind=cfg.CONTROLLED_XR_, phase=cfg.DEF_PHASE, qid=[q0, q1])
-        return self
-
-    def ch(self, q0, q1):
-        """
-        operate CH gate (controlled H gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id (control qubit).
-        q1 : int
-            qubit id (target qubit).
-
-        Returns
-        -------
-        self : instance of DensOp
-
-        """
-        densop_operate_qgate(self, kind=cfg.CONTROLLED_H, phase=cfg.DEF_PHASE, qid=[q0, q1])
-        return self
-
-    def cs(self, q0, q1):
-        """
-        operate CS gate (controlled S gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id (control qubit).
-        q1 : int
-            qubit id (target qubit).
-
-        Returns
-        -------
-        self : instance of DensOp
-
-        """
-        densop_operate_qgate(self, kind=cfg.CONTROLLED_S, phase=cfg.DEF_PHASE, qid=[q0, q1])
-        return self
-
-    def cs_dg(self, q0, q1):
-        """
-        operate CS dagger gate (controlled S dagger gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id (control qubit).
-        q1 : int
-            qubit id (target qubit).
-
-        Returns
-        -------
-        self : instance of DensOp
-
-        """
-        densop_operate_qgate(self, kind=cfg.CONTROLLED_S_, phase=cfg.DEF_PHASE, qid=[q0, q1])
-        return self
-
-    def ct(self, q0, q1):
-        """
-        operate CT gate (controlled T gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id (control qubit).
-        q1 : int
-            qubit id (target qubit).
-
-        Returns
-        -------
-        self : instance of DensOp
-
-        """
-        densop_operate_qgate(self, kind=cfg.CONTROLLED_T, phase=cfg.DEF_PHASE, qid=[q0, q1])
-        return self
-
-    def ct_dg(self, q0, q1):
-        """
-        operate CT dagger gate (controlled T dagger gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id (control qubit).
-        q1 : int
-            qubit id (target qubit).
-
-        Returns
-        -------
-        self : instance of DensOp
-
-        """
-        densop_operate_qgate(self, kind=cfg.CONTROLLED_T_, phase=cfg.DEF_PHASE, qid=[q0, q1])
-        return self
-
-    def sw(self, q0, q1):
-        """
-        swap gate
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id
-        q1 : int
-            qubit id
-
-        Returns
-        -------
-        self : instance of DensOp
-
-        """
-        densop_operate_qgate(self, kind=cfg.SWAP_QUBITS, phase=cfg.DEF_PHASE, qid=[q0, q1])
-        return self
-
-    def cp(self, q0, q1, phase=cfg.DEF_PHASE):
-        """
-        operate CP gate (controlled P gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id (control qubit).
-        q1 : int
-            qubit id (target qubit).
-
-        Returns
-        -------
-        self : instance of DensOp
-
-        """
-        densop_operate_qgate(self, kind=cfg.CONTROLLED_P, phase=phase, qid=[q0, q1])
-        return self
-
-    def crx(self, q0, q1, phase=cfg.DEF_PHASE):
-        """
-        operate CRX gate (controlled RX gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id (control qubit).
-        q1 : int
-            qubit id (target qubit).
-        phase : float
-            rotation angle (unit of angle is PI radian).
-
-        Returns
-        -------
-        self : instance of DensOp
-
-        """
-        densop_operate_qgate(self, kind=cfg.CONTROLLED_RX, phase=phase, qid=[q0, q1])
-        return self
-
-    def cry(self, q0, q1, phase=cfg.DEF_PHASE):
-        """
-        operate CRY gate (controlled RY gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id (control qubit).
-        q1 : int
-            qubit id (target qubit).
-        phase : float
-            rotation angle (unit of angle is PI radian).
-
-        Returns
-        -------
-        self : instance of DensOp
-
-        """
-        densop_operate_qgate(self, kind=cfg.CONTROLLED_RY, phase=phase, qid=[q0, q1])
-        return self
-
-    def crz(self, q0, q1, phase=cfg.DEF_PHASE):
-        """
-        operate CRZ gate (controlled RZ gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id (control qubit).
-        q1 : int
-            qubit id (target qubit).
-        phase : float
-            rotation angle (unit of angle is PI radian).
-
-        Returns
-        -------
-        self : instance of DensOp
-
-        """
-        densop_operate_qgate(self, kind=cfg.CONTROLLED_RZ, phase=phase, qid=[q0, q1])
-        return self
-
-    def rxx(self, q0, q1, phase=cfg.DEF_PHASE):
-        """
-        operate Rxx gate.
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id (control qubit).
-        q1 : int
-            qubit id (target qubit).
-        phase : float
-            rotation angle (unit of angle is PI radian).
-
-        Returns
-        -------
-        self : instance of DensOp
-
-        """
-        densop_operate_qgate(self, kind=cfg.ROTATION_XX, phase=phase, qid=[q0, q1])
-        return self
-
-    def ryy(self, q0, q1, phase=cfg.DEF_PHASE):
-        """
-        operate Rxx gate.
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id (control qubit).
-        q1 : int
-            qubit id (target qubit).
-        phase : float
-            rotation angle (unit of angle is PI radian).
-
-        Returns
-        -------
-        self : instance of DensOp
-
-        """
-        densop_operate_qgate(self, kind=cfg.ROTATION_YY, phase=phase, qid=[q0, q1])
-        return self
-
-    def rzz(self, q0, q1, phase=cfg.DEF_PHASE):
-        """
-        operate Rzz gate.
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id (control qubit).
-        q1 : int
-            qubit id (target qubit).
-        phase : float
-            rotation angle (unit of angle is PI radian).
-
-        Returns
-        -------
-        self : instance of DensOp
-
-        """
-        densop_operate_qgate(self, kind=cfg.ROTATION_ZZ, phase=phase, qid=[q0, q1])
-        return self
-
-    # 3-qubit gate
-
-    def ccx(self, q0, q1, q2):
-        """
-        operate CCX gate (toffoli gate, controlled controlled X gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id (control qubit).
-        q1 : int
-            qubit id (control qubit).
-        q2 : int
-            qubit id (target qubit).
-
-        Returns
-        -------
-        self : instance of DensOp
-
-        """
-        self.cxr(q1, q2).cx(q0, q1).cxr_dg(q1, q2).cx(q0, q1).cxr(q0, q2)
-        return self
-
-    def csw(self, q0, q1, q2):
-        """
-        operate CSW gate (fredkin gate, controlled swap gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id (control qubit).
-        q1 : int
-            qubit id (swap qubit).
-        q2 : int
-            qubit id (swap qubit).
-
-        Returns
-        -------
-        self : instance of DensOp
-
-        """
-        self.cx(q2, q1).ccx(q0, q1, q2).cx(q2, q1)
-        return self
-
-    # other gate
-
-    def mcx(self, qid=None):
-        """
-        operate MCX gate (multi-controlled X gate).
-
-        Parameters
-        ----------
-        qid : list of int
-            qubit id list [control, control, ... , control, target]
-
-        Returns
-        -------
-        self : instance of DensOp
-
-        """
-        densop_mcx(self, qid)
-        return self
-
-    def operate(self, pp=None, ctrl=None, qctrl=None):
-        """
-        operate unitary operator to density operator.
-
-        Parameters
-        ----------
-        pp : instance of PauliProduct
-            pauli product to operate
-        ctrl : int
-            contoroll qubit id for controlled pauli product
-            (this option will be removed near future)
-        qctrl : int
-            contoroll qubit id for controlled pauli product
-
-        Returns
-        -------
-        self : instance of DensOp
-            density operator after operation
-
-        """
-        pauli_list = pp.pauli_list
-        qid = pp.qid
-        factor = pp.factor
-
-        if ctrl is None:
-            ctrl = qctrl
-
-        if ctrl is None:
-            for q, pauli in zip(qid, pauli_list):
-                if pauli == 'X':
-                    self.x(q)
-                elif pauli == 'Y':
-                    self.y(q)
-                elif pauli == 'Z':
-                    self.z(q)
-                else:
-                    continue
+        if qctrl is None:
+            densop_operate_qcirc(self, qcirc=qcirc)
         else:
-            if ctrl in qid:
-                raise ValueError("controll and target qubit id conflict")
-
-            for q, pauli in zip(qid, pauli_list):
-                if pauli == 'X':
-                    self.cx(ctrl, q)
-                elif pauli == 'Y':
-                    self.cy(ctrl, q)
-                elif pauli == 'Z':
-                    self.cz(ctrl, q)
-                else:
-                    continue
-
-            if factor == -1.+0.j:
-                self.z(ctrl)
-            elif factor == 0.+1.j:
-                self.s(ctrl)
-            elif factor == 0.-1.j:
-                self.s_dg(ctrl)
+            qc_qctrl = qcirc.add_control(qctrl=qctrl)
+            densop_operate_qcirc(self, qcirc=qc_qctrl)
 
         return self
 
