@@ -10,12 +10,9 @@ import tensornetwork as tn
 from tensornetwork import FiniteMPS
 
 import qlazy.config as cfg
-
-def gray_code(n):
-    """ gray code generator (for mcx method) """
-
-    for k in range(2**n):
-        yield k^(k>>1)
+from qlazy.util import is_unitary_gate, get_qgate_qubit_num
+from qlazy.QObject import QObject
+from qlazy.lib.mpstate_func import mps_operate_qcirc
 
 class MDataMPState:
     """ Measured Data for MPState
@@ -40,7 +37,7 @@ class MDataMPState:
         self.qubit_num = qubit_num
 
 
-class MPState(FiniteMPS):
+class MPState(FiniteMPS, QObject):
     """ Matrix Product State
 
     Attributes
@@ -675,709 +672,61 @@ class MPState(FiniteMPS):
 
         return self
 
-    # 1-qubit gate
-
-    def x(self, q0):
+    def reset_qubits(self, qid=None):
         """
-        operate X gate.
+        reset to |00..0> state.
 
         Parameters
         ----------
-        q0 : int
-            qubit id.
+        qid : list, default - qubit id's list for all of the qubits
+            qubit id's list to reset.
 
         Returns
         -------
         self : instance of MPState
-
-        """
-        self.operate_1qubit_gate('x', q0)
-        return self
-
-    def y(self, q0):
-        """
-        operate Y gate.
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id.
-
-        Returns
-        -------
-        self : instance of MPState
-
-        """
-        self.operate_1qubit_gate('y', q0)
-        return self
-
-    def z(self, q0):
-        """
-        operate Z gate.
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id.
-
-        Returns
-        -------
-        self : instance of MPState
-
-        """
-        self.operate_1qubit_gate('z', q0)
-        return self
-
-    def xr(self, q0):
-        """
-        operate root X gate.
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id.
-
-        Returns
-        -------
-        self : instance of MPState
-
-        """
-        self.operate_1qubit_gate('xr', q0)
-        return self
-
-    def xr_dg(self, q0):
-        """
-        operate root X dagger gate
-        (hermmitian conjugate of root X gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id.
-
-        Returns
-        -------
-        self : instance of MPState
-
-        """
-        self.operate_1qubit_gate('xr_dg', q0)
-        return self
-
-    def h(self, q0):
-        """
-        operate H gate (hadamard gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id.
-
-        Returns
-        -------
-        self : instance of MPState
-
-        """
-        self.operate_1qubit_gate('h', q0)
-        return self
-
-    def s(self, q0):
-        """
-        operate S gate.
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id.
-
-        Returns
-        -------
-        self : instance of MPState
-
-        """
-        self.operate_1qubit_gate('s', q0)
-        return self
-
-    def s_dg(self, q0):
-        """
-        operate S dagger gate (hermitian conjugate of S gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id.
-
-        Returns
-        -------
-        self : instance of MPState
-
-        """
-        self.operate_1qubit_gate('s_dg', q0)
-        return self
-
-    def t(self, q0):
-        """
-        operate T gate.
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id.
-
-        Returns
-        -------
-        self : instance of MPState
-
-        """
-        self.operate_1qubit_gate('t', q0)
-        return self
-
-    def t_dg(self, q0):
-        """
-        operate T dagger gate (hermitian conjugate of T gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id.
-
-        Returns
-        -------
-        self : instance of MPState
-
-        """
-        self.operate_1qubit_gate('t_dg', q0)
-        return self
-
-    def rx(self, q0, phase=0.0):
-        """
-        operate RX gate (rotation around X-axis).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id.
-        phase : float
-            rotation angle (unit of angle is PI radian).
-
-        Returns
-        -------
-        self : instance of QState
-
-        """
-        self.operate_1qubit_gate('rx', q0, para=phase)
-        return self
-
-    def ry(self, q0, phase=0.0):
-        """
-        operate RY gate (rotation around Y-axis).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id.
-        phase : float
-            rotation angle (unit of angle is PI radian).
-
-        Returns
-        -------
-        self : instance of QState
-
-        """
-        self.operate_1qubit_gate('ry', q0, para=phase)
-        return self
-
-    def rz(self, q0, phase=0.0):
-        """
-        operate RZ gate (rotation around Z-axis).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id.
-        phase : float
-            rotation angle (unit of angle is PI radian).
-
-        Returns
-        -------
-        self : instance of QState
-
-        """
-        self.operate_1qubit_gate('rz', q0, para=phase)
-        return self
-
-    def p(self, q0, phase=0.0):
-        """
-        operate P gate (phase shift gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id.
-        phase : float
-            rotation angle (unit of angle is PI radian).
-
-        Returns
-        -------
-        self : instance of QState
+            matrix product state.
 
         Notes
         -----
-        matrix expression is following...
-        | 1.0 0.0             |
-        | 0.0 exp(i*phase*PI) |
+        If 'qid' is set, specified qubits are reset after
+        measurement. So if the specified qubits are entangled with the
+        remaining qubits, output quantum state is probabilistic. If no
+        qubits are set, all qubits are zero reset.
 
         """
-        self.operate_1qubit_gate('p', q0, para=phase)
-        return self
+        if qid is None or qid == []:
+            qid = list(range(self.__qubit_num))
 
-    # 2-qubit gate
+        proj_gate = [np.array([[1.+0.j, 0.+0.j],
+                               [0.+0.j, 0.+0.j]]),
+                     np.array([[0.+0.j, 0.+0.j],
+                               [0.+0.j, 1.+0.j]])]
 
-    def cx(self, q0, q1):
-        """
-        operate CX gate (controlled X gate, controlled NOT gate, CNOT gate).
+        self.canonicalize(normalize=True)
 
-        Parameters
-        ----------
-        q0 : int
-            qubit id (control qubit).
-        q1 : int
-            qubit id (target qubit).
-
-        Returns
-        -------
-        self : instance of QState
-
-        """
-        self.operate_2qubit_gate('cx', q0, q1)
-        return self
-
-    def cy(self, q0, q1):
-        """
-        operate CY gate (controlled X gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id (control qubit).
-        q1 : int
-            qubit id (target qubit).
-
-        Returns
-        -------
-        self : instance of QState
-
-        """
-        self.operate_2qubit_gate('cy', q0, q1)
-        return self
-
-    def cz(self, q0, q1):
-        """
-        operate CZ gate (controlled Z gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id (control qubit).
-        q1 : int
-            qubit id (target qubit).
-
-        Returns
-        -------
-        self : instance of QState
-
-        """
-        self.operate_2qubit_gate('cz', q0, q1)
-        return self
-
-    def cxr(self, q0, q1):
-        """
-        operate CXR gate (controlled root X gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id (control qubit).
-        q1 : int
-            qubit id (target qubit).
-
-        Returns
-        -------
-        self : instance of QState
-
-        """
-        self.operate_2qubit_gate('cxr', q0, q1)
-        return self
-
-    def cxr_dg(self, q0, q1):
-        """
-        operate CXR dagger gate (controlled XR dagger gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id (control qubit).
-        q1 : int
-            qubit id (target qubit).
-
-        Returns
-        -------
-        self : instance of QState
-
-        """
-        self.operate_2qubit_gate('cxr_dg', q0, q1)
-        return self
-
-    def ch(self, q0, q1):
-        """
-        operate CH gate (controlled H gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id (control qubit).
-        q1 : int
-            qubit id (target qubit).
-
-        Returns
-        -------
-        self : instance of QState
-
-        """
-        self.operate_2qubit_gate('ch', q0, q1)
-        return self
-
-    def cs(self, q0, q1):
-        """
-        operate CS gate (controlled S gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id (control qubit).
-        q1 : int
-            qubit id (target qubit).
-
-        Returns
-        -------
-        self : instance of QState
-
-        """
-        self.operate_2qubit_gate('cs', q0, q1)
-        return self
-
-    def cs_dg(self, q0, q1):
-        """
-        operate CS dagger gate (controlled S dagger gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id (control qubit).
-        q1 : int
-            qubit id (target qubit).
-
-        Returns
-        -------
-        self : instance of QState
-
-        """
-        self.operate_2qubit_gate('cs_dg', q0, q1)
-        return self
-
-    def ct(self, q0, q1):
-        """
-        operate CT gate (controlled T gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id (control qubit).
-        q1 : int
-            qubit id (target qubit).
-
-        Returns
-        -------
-        self : instance of QState
-
-        """
-        self.operate_2qubit_gate('ct', q0, q1)
-        return self
-
-    def ct_dg(self, q0, q1):
-        """
-        operate CT dagger gate (controlled T dagger gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id (control qubit).
-        q1 : int
-            qubit id (target qubit).
-
-        Returns
-        -------
-        self : instance of QState
-
-        """
-        self.operate_2qubit_gate('ct_dg', q0, q1)
-        return self
-
-    def sw(self, q0, q1):
-        """
-        swap gate
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id
-        q1 : int
-            qubit id
-
-        Returns
-        -------
-        self : instance of QState
-
-        """
-        self.operate_2qubit_gate('sw', q0, q1)
-        return self
-
-    def cp(self, q0, q1, phase=0.0):
-        """
-        operate CP gate (controlled P gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id (control qubit).
-        q1 : int
-            qubit id (target qubit).
-
-        Returns
-        -------
-        self : instance of QState
-
-        """
-        self.operate_2qubit_gate('cp', q0, q1, para=phase)
-        return self
-
-    def crx(self, q0, q1, phase=0.0):
-        """
-        operate CRX gate (controlled RX gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id (control qubit).
-        q1 : int
-            qubit id (target qubit).
-        phase : float
-            rotation angle (unit of angle is PI radian).
-
-        Returns
-        -------
-        self : instance of QState
-
-        """
-        self.operate_2qubit_gate('crx', q0, q1, para=phase)
-        return self
-
-    def cry(self, q0, q1, phase=0.0):
-        """
-        operate CRY gate (controlled RY gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id (control qubit).
-        q1 : int
-            qubit id (target qubit).
-        phase : float
-            rotation angle (unit of angle is PI radian).
-
-        Returns
-        -------
-        self : instance of QState
-
-        """
-        self.operate_2qubit_gate('cry', q0, q1, para=phase)
-        return self
-
-    def crz(self, q0, q1, phase=0.0):
-        """
-        operate CRZ gate (controlled RZ gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id (control qubit).
-        q1 : int
-            qubit id (target qubit).
-        phase : float
-            rotation angle (unit of angle is PI radian).
-
-        Returns
-        -------
-        self : instance of QState
-
-        """
-        self.operate_2qubit_gate('crz', q0, q1, para=phase)
-        return self
-
-    def rxx(self, q0, q1, phase=0.0):
-        """
-        operate Rxx gate.
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id (control qubit).
-        q1 : int
-            qubit id (target qubit).
-        phase : float
-            rotation angle (unit of angle is PI radian).
-
-        Returns
-        -------
-        self : instance of QState
-
-        """
-        self.operate_2qubit_gate('rxx', q0, q1, para=phase)
-        return self
-
-    def ryy(self, q0, q1, phase=0.0):
-        """
-        operate Ryy gate.
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id (control qubit).
-        q1 : int
-            qubit id (target qubit).
-        phase : float
-            rotation angle (unit of angle is PI radian).
-
-        Returns
-        -------
-        self : instance of QState
-
-        """
-        self.operate_2qubit_gate('ryy', q0, q1, para=phase)
-        return self
-
-    def rzz(self, q0, q1, phase=0.0):
-        """
-        operate Rxx gate.
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id (control qubit).
-        q1 : int
-            qubit id (target qubit).
-        phase : float
-            rotation angle (unit of angle is PI radian).
-
-        Returns
-        -------
-        self : instance of QState
-
-        """
-        self.operate_2qubit_gate('rzz', q0, q1, para=phase)
-        return self
-
-    # 3-qubit gate
-
-    def ccx(self, q0, q1, q2):
-        """
-        operate CCX gate (toffoli gate, controlled controlled X gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id (control qubit).
-        q1 : int
-            qubit id (control qubit).
-        q2 : int
-            qubit id (target qubit).
-
-        Returns
-        -------
-        self : instance of QState
-
-        """
-        self.cxr(q1, q2).cx(q0, q1).cxr_dg(q1, q2).cx(q0, q1).cxr(q0, q2)
-        return self
-
-    def csw(self, q0, q1, q2):
-        """
-        operate CSW gate (fredkin gate, controlled swap gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id (control qubit).
-        q1 : int
-            qubit id (swap qubit).
-        q2 : int
-            qubit id (swap qubit).
-
-        Returns
-        -------
-        self : instance of QState
-
-        """
-        self.cx(q2, q1).ccx(q0, q1, q2).cx(q2, q1)
-        return self
-
-    # other gate
-
-    def mcx(self, qid=None):
-        """
-        operate MCX gate (multi-controlled X gate).
-
-        Parameters
-        ----------
-        qid : list of int
-            qubit id list [control, control, ... , control, target]
-
-        Returns
-        -------
-        self : instance of QState
-
-        """
-        if qid is None:
-            raise ValueError("qid must be set.")
-
-        # controled and target register
-        qid_ctr = qid[:-1]
-        qid_tar = qid[-1]
-
-        # hadamard
-        self.h(qid_tar)
-
-        # controlled-RZ(psi), psi=pi/(2**(bitnum-1))
-        bitnum = len(qid_ctr)
-        psi = 1.0/(2**(bitnum-1)) # unit=pi(radian)
-        gray_pre = 0
-        for gray in gray_code(bitnum):
-            if gray == 0:
+        for q in qid:
+            self.position(q)
+            prob = self.__probability(q=q)
+            if abs(prob[0] - 1.0) < cfg.EPS:
+                measured_value = 0
+                continue
+            elif abs(prob[1] - 1.0) < cfg.EPS:
+                measured_value = 1
+                self.x(q)
                 continue
-            msb = len(str(bin(gray)))-3
-            chb = len(str(bin(gray^gray_pre)))-3
-            if gray != 1:
-                if chb == msb:
-                    chb -= 1
-                self.cx(qid_ctr[chb], qid_ctr[msb])
-            self.cp(qid_ctr[msb], qid_tar, phase=psi)
-            psi = -psi
-            gray_pre = gray
 
-        # hadamard
-        self.h(qid_tar)
+            if random.random() < prob[0]:
+                measured_value = 0
+                self.apply_one_site_gate(gate=proj_gate[measured_value], site=q)
+                self.canonicalize(normalize=True)
+            else:
+                measured_value = 1
+                self.apply_one_site_gate(gate=proj_gate[measured_value], site=q)
+                self.canonicalize(normalize=True)
+                self.x(q)
 
         return self
-    
+
     def __probability(self, q):
 
         z_gate = self.__get_gate_array('z')
@@ -1530,125 +879,6 @@ class MPState(FiniteMPS):
         mval = md.last
         return mval
         
-    def reset(self, qid=None):
-        """
-        reset to |00..0> state.
-
-        Parameters
-        ----------
-        qid : list, default - qubit id's list for all of the qubits
-            qubit id's list to reset.
-
-        Returns
-        -------
-        self : instance of MPState
-            matrix product state.
-
-        Notes
-        -----
-        If 'qid' is set, specified qubits are reset after
-        measurement. So if the specified qubits are entangled with the
-        remaining qubits, output quantum state is probabilistic. If no
-        qubits are set, all qubits are zero reset.
-
-        """
-        if qid is None or qid == []:
-            qid = list(range(self.__qubit_num))
-
-        proj_gate = [np.array([[1.+0.j, 0.+0.j],
-                               [0.+0.j, 0.+0.j]]),
-                     np.array([[0.+0.j, 0.+0.j],
-                               [0.+0.j, 1.+0.j]])]
-
-        self.canonicalize(normalize=True)
-
-        for q in qid:
-            self.position(q)
-            prob = self.__probability(q=q)
-            if abs(prob[0] - 1.0) < cfg.EPS:
-                measured_value = 0
-                continue
-            elif abs(prob[1] - 1.0) < cfg.EPS:
-                measured_value = 1
-                self.x(q)
-                continue
-
-            if random.random() < prob[0]:
-                measured_value = 0
-                self.apply_one_site_gate(gate=proj_gate[measured_value], site=q)
-                self.canonicalize(normalize=True)
-            else:
-                measured_value = 1
-                self.apply_one_site_gate(gate=proj_gate[measured_value], site=q)
-                self.canonicalize(normalize=True)
-                self.x(q)
-
-        return self
-
-    def operate(self, pp=None, ctrl=None, qctrl=None):
-        """
-        operate unitary operator to quantum state.
-
-        Parameters
-        ----------
-        pp : instance of PauliProduct
-            pauli product to operate
-        ctrl : int
-            contoroll qubit id for controlled pauli product
-            (this option will be removed near future)
-        qctrl : int
-            contoroll qubit id for controlled pauli product
-
-        Returns
-        -------
-        self : instance of QState
-            quantum state after operation
-
-        See Also
-        --------
-        PauliProduct class (PauliProduct.py)
-
-        """
-        pauli_list = pp.pauli_list
-        qid = pp.qid
-        factor = pp.factor
-
-        if ctrl is None:
-            ctrl = qctrl
-
-        if ctrl is None:
-            for q, pauli in zip(qid, pauli_list):
-                if pauli == 'X':
-                    self.x(q)
-                elif pauli == 'Y':
-                    self.y(q)
-                elif pauli == 'Z':
-                    self.z(q)
-                else:
-                    continue
-        else:
-            if ctrl in qid:
-                raise ValueError("controll and target qubit id conflict")
-
-            for q, pauli in zip(qid, pauli_list):
-                if pauli == 'X':
-                    self.cx(ctrl, q)
-                elif pauli == 'Y':
-                    self.cy(ctrl, q)
-                elif pauli == 'Z':
-                    self.cz(ctrl, q)
-                else:
-                    continue
-
-            if factor == -1.+0.j:
-                self.z(ctrl)
-            elif factor == 0.+1.j:
-                self.s(ctrl)
-            elif factor == 0.-1.j:
-                self.s_dg(ctrl)
-
-        return self
-
     def expect(self, observable=None):
         """
         get the expectation value for observable under the matrix product state.
@@ -1686,3 +916,76 @@ class MPState(FiniteMPS):
             expect_value += (weight * self.inpro(mps))
 
         return expect_value
+
+    # operate gate
+
+    def operate_gate(self, kind=None, qid=None, phase=0.0, **kwargs):
+        """
+        operate gate
+
+        Parameters
+        ----------
+        kind : int
+            kind of the gate
+        qid : list
+            quantum id list
+        phase : float
+            phase for rotation gate
+
+        Returns
+        -------
+        self : instance of MPState
+            quantum state
+
+        """
+        gstr = cfg.GATE_STRING[kind]
+        if kind == cfg.RESET:
+            self.reset_qubits(qid)
+        elif is_unitary_gate(kind):
+            if get_qgate_qubit_num(kind) == 1:
+                q0 = qid[0]
+                self.operate_1qubit_gate(gstr, q0, para=phase)
+            elif get_qgate_qubit_num(kind) == 2:
+                q0, q1 = qid[0], qid[1]
+                self.operate_2qubit_gate(gstr, q0, q1, para=phase)
+            else:
+                raise ValueError("length of qid must be 1 or 2.")
+        else:
+            raise ValueError("gate: {} is not supported.".format(cfg.GATE_STRING[kind]))
+        return self
+
+    # operate quantum circuit
+
+    def operate_qcirc(self, qcirc, qctrl=None):
+        """
+        operate quantum circuit
+
+        Parameters
+        ----------
+        qcirc : instance of QCirc
+            quantum circuit
+        qctrl : int
+            control qubit id
+
+        Returns
+        -------
+        self : instance of MPState
+            quantum state after executing the quantum circuit
+
+        Notes
+        -----
+        The quantum circut must be unintary.
+
+        """
+        if qcirc.is_unitary() is False:
+            raise ValueError("qcirc must be clifford quantum circuit.")
+        if self.qubit_num < qcirc.qubit_num:
+            raise ValueError("qubit number of quantum state must be equal or larger than the quantum circuit size.")
+
+        if qctrl is None:
+            mps_operate_qcirc(self, cmem=None, qcirc=qcirc, shots=1, cid=None)
+        else:
+            qc_qctrl = qcirc.add_control(qctrl=qctrl)
+            mps_operate_qcirc(self, cmem=None, qcirc=qc_qctrl, shots=1, cid=None)
+
+        return self
