@@ -65,6 +65,8 @@ class Observable:
         ex) "-2.0+Z_0*Z_1+X_0+X_1" for 2-qubit system
     weighted_pp_list : list of dict ({'weight':weight, 'pp':pauli_product})
         weighted pauli product list
+    qubit_num : int
+        total qubit number
 
     """
     def __init__(self, string=None):
@@ -162,7 +164,7 @@ class Observable:
 
         return ans
 
-    def __neq__(self, ob):
+    def __ne__(self, ob):
         """
         Parameters
         ----------
@@ -225,7 +227,7 @@ class Observable:
             if wpp['weight'] != 0.0:
                 out.add_wpp(weight=wpp['weight'], pp=wpp['pp'])
 
-        if isinstance(other, Observable):
+        if isinstance(other, self.__class__):
             for wpp in other.weighted_pp_list:
                 if wpp['weight'] != 0.0:
                     out.add_wpp(weight=wpp['weight'], pp=wpp['pp'])
@@ -274,7 +276,7 @@ class Observable:
             if wpp['weight'] != 0.0:
                 out.add_wpp(weight=wpp['weight'], pp=wpp['pp'])
 
-        if isinstance(other, Observable):
+        if isinstance(other, self.__class__):
             for wpp in other.weighted_pp_list:
                 if wpp['weight'] != 0.0:
                     out.add_wpp(weight=-wpp['weight'], pp=wpp['pp'])
@@ -321,7 +323,7 @@ class Observable:
             observable (result)
 
         """
-        if isinstance(other, Observable):
+        if isinstance(other, self.__class__):
             out = Observable()
             for wpp_1 in self.weighted_pp_list:
                 for wpp_2 in other.weighted_pp_list:
@@ -676,3 +678,27 @@ class Observable:
                 self.weighted_pp_list[i]['weight'] = w
 
         return self
+
+    @property
+    def qubit_num(self):
+        return self.get_qubit_num()
+
+    def get_qubit_num(self):
+        """
+        get the total qubit number considerd
+
+        Parameters
+        ----------
+        None
+
+        Returns
+        -------
+        qubit_num : inst
+            total qubit number considerd by the observable.
+
+        """
+        qubit_num = 0
+        for wpp in self.weighted_pp_list:
+            qubit_num = max(qubit_num, wpp['pp'].get_qubit_num())
+
+        return qubit_num
