@@ -5,10 +5,11 @@ import random
 import numpy as np
 
 import qlazy.config as cfg
-from qlazy.lib.qstate_mcx import qstate_mcx
 from qlazy.gpu import is_gpu_supported_lib
+from qlazy.util import is_unitary_gate
+from qlazy.QObject import QObject
 
-class QState(ctypes.Structure):
+class QState(ctypes.Structure, QObject):
     """ Quantum State
 
     Attributes
@@ -93,31 +94,6 @@ class QState(ctypes.Structure):
 
         return str(self.get_amp())
 
-    def reset(self, qid=None):
-        """
-        reset to |00..0> state.
-
-        Parameters
-        ----------
-        qid : list, default - qubit id's list for all of the qubits
-            qubit id's list to reset.
-
-        Returns
-        -------
-        self : instance of QState
-            quantum state.
-
-        Notes
-        -----
-        If 'qid' is set, specified qubits are reset after
-        measurement. So if the specified qubits are entangled with the
-        remaining qubits, output quantum state is probabilistic. If no
-        qubits are set, all qubits are zero reset.
-
-        """
-        qstate_reset(self, qid=qid)
-        return self
-
     @classmethod
     def add_method(cls, method):
         """
@@ -382,8 +358,6 @@ class QState(ctypes.Structure):
         c[11] = +0.7071+0.0000*i : 0.5000 |++++++
 
         """
-        # qstate_print(self, qid, nonzero)
-
         if qid is None or len(qid) == 0:
             digits = self.qubit_num
         else:
@@ -736,680 +710,6 @@ class QState(ctypes.Structure):
         coef = self.__schmidt_decomp(qid_0=qid_0, qid_1=qid_1)[0]
         return coef
 
-    # 1-qubit gate
-
-    def x(self, q0):
-        """
-        operate X gate.
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id.
-
-        Returns
-        -------
-        self : instance of QState
-
-        """
-        qstate_operate_qgate(self, kind=cfg.PAULI_X, phase=cfg.DEF_PHASE, qid=[q0])
-        return self
-
-    def y(self, q0):
-        """
-        operate Y gate.
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id.
-
-        Returns
-        -------
-        self : instance of QState
-
-        """
-        qstate_operate_qgate(self, kind=cfg.PAULI_Y, phase=cfg.DEF_PHASE, qid=[q0])
-        return self
-
-    def z(self, q0):
-        """
-        operate Z gate.
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id.
-
-        Returns
-        -------
-        self : instance of QState
-
-        """
-        qstate_operate_qgate(self, kind=cfg.PAULI_Z, phase=cfg.DEF_PHASE, qid=[q0])
-        return self
-
-    def xr(self, q0):
-        """
-        operate root X gate.
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id.
-
-        Returns
-        -------
-        self : instance of QState
-
-        """
-        qstate_operate_qgate(self, kind=cfg.ROOT_PAULI_X, phase=cfg.DEF_PHASE, qid=[q0])
-        return self
-
-    def xr_dg(self, q0):
-        """
-        operate root X dagger gate
-        (hermmitian conjugate of root X gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id.
-
-        Returns
-        -------
-        self : instance of QState
-
-        """
-        qstate_operate_qgate(self, kind=cfg.ROOT_PAULI_X_, phase=cfg.DEF_PHASE, qid=[q0])
-        return self
-
-    def h(self, q0):
-        """
-        operate H gate (hadamard gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id.
-
-        Returns
-        -------
-        self : instance of QState
-
-        """
-        qstate_operate_qgate(self, kind=cfg.HADAMARD, phase=cfg.DEF_PHASE, qid=[q0])
-        return self
-
-    def s(self, q0):
-        """
-        operate S gate.
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id.
-
-        Returns
-        -------
-        self : instance of QState
-
-        """
-        qstate_operate_qgate(self, kind=cfg.PHASE_SHIFT_S, phase=cfg.DEF_PHASE, qid=[q0])
-        return self
-
-    def s_dg(self, q0):
-        """
-        operate S dagger gate (hermitian conjugate of S gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id.
-
-        Returns
-        -------
-        self : instance of QState
-
-        """
-        qstate_operate_qgate(self, kind=cfg.PHASE_SHIFT_S_, phase=cfg.DEF_PHASE, qid=[q0])
-        return self
-
-    def t(self, q0):
-        """
-        operate T gate.
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id.
-
-        Returns
-        -------
-        self : instance of QState
-
-        """
-        qstate_operate_qgate(self, kind=cfg.PHASE_SHIFT_T, phase=cfg.DEF_PHASE, qid=[q0])
-        return self
-
-    def t_dg(self, q0):
-        """
-        operate T dagger gate (hermitian conjugate of T gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id.
-
-        Returns
-        -------
-        self : instance of QState
-
-        """
-        qstate_operate_qgate(self, kind=cfg.PHASE_SHIFT_T_, phase=cfg.DEF_PHASE, qid=[q0])
-        return self
-
-    def rx(self, q0, phase=cfg.DEF_PHASE):
-        """
-        operate RX gate (rotation around X-axis).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id.
-        phase : float
-            rotation angle (unit of angle is PI radian).
-
-        Returns
-        -------
-        self : instance of QState
-
-        """
-        qstate_operate_qgate(self, kind=cfg.ROTATION_X, phase=phase, qid=[q0])
-        return self
-
-    def ry(self, q0, phase=cfg.DEF_PHASE):
-        """
-        operate RY gate (rotation around Y-axis).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id.
-        phase : float
-            rotation angle (unit of angle is PI radian).
-
-        Returns
-        -------
-        self : instance of QState
-
-        """
-        qstate_operate_qgate(self, kind=cfg.ROTATION_Y, phase=phase, qid=[q0])
-        return self
-
-    def rz(self, q0, phase=cfg.DEF_PHASE):
-        """
-        operate RZ gate (rotation around Z-axis).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id.
-        phase : float
-            rotation angle (unit of angle is PI radian).
-
-        Returns
-        -------
-        self : instance of QState
-
-        """
-        qstate_operate_qgate(self, kind=cfg.ROTATION_Z, phase=phase, qid=[q0])
-        return self
-
-    def p(self, q0, phase=cfg.DEF_PHASE):
-        """
-        operate P gate (phase shift gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id.
-        phase : float
-            rotation angle (unit of angle is PI radian).
-
-        Returns
-        -------
-        self : instance of QState
-
-        Notes
-        -----
-        matrix expression is following...
-        | 1.0 0.0             |
-        | 0.0 exp(i*phase*PI) |
-
-        """
-        qstate_operate_qgate(self, kind=cfg.PHASE_SHIFT, phase=phase, qid=[q0])
-        return self
-
-    # 2-qubit gate
-
-    def cx(self, q0, q1):
-        """
-        operate CX gate (controlled X gate, controlled NOT gate, CNOT gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id (control qubit).
-        q1 : int
-            qubit id (target qubit).
-
-        Returns
-        -------
-        self : instance of QState
-
-        """
-        qstate_operate_qgate(self, kind=cfg.CONTROLLED_X, phase=cfg.DEF_PHASE, qid=[q0, q1])
-        return self
-
-    def cy(self, q0, q1):
-        """
-        operate CY gate (controlled X gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id (control qubit).
-        q1 : int
-            qubit id (target qubit).
-
-        Returns
-        -------
-        self : instance of QState
-
-        """
-        qstate_operate_qgate(self, kind=cfg.CONTROLLED_Y, phase=cfg.DEF_PHASE, qid=[q0, q1])
-        return self
-
-    def cz(self, q0, q1):
-        """
-        operate CZ gate (controlled Z gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id (control qubit).
-        q1 : int
-            qubit id (target qubit).
-
-        Returns
-        -------
-        self : instance of QState
-
-        """
-        qstate_operate_qgate(self, kind=cfg.CONTROLLED_Z, phase=cfg.DEF_PHASE, qid=[q0, q1])
-        return self
-
-    def cxr(self, q0, q1):
-        """
-        operate CXR gate (controlled root X gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id (control qubit).
-        q1 : int
-            qubit id (target qubit).
-
-        Returns
-        -------
-        self : instance of QState
-
-        """
-        qstate_operate_qgate(self, kind=cfg.CONTROLLED_XR, phase=cfg.DEF_PHASE, qid=[q0, q1])
-        return self
-
-    def cxr_dg(self, q0, q1):
-        """
-        operate CXR dagger gate (controlled XR dagger gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id (control qubit).
-        q1 : int
-            qubit id (target qubit).
-
-        Returns
-        -------
-        self : instance of QState
-
-        """
-        qstate_operate_qgate(self, kind=cfg.CONTROLLED_XR_, phase=cfg.DEF_PHASE, qid=[q0, q1])
-        return self
-
-    def ch(self, q0, q1):
-        """
-        operate CH gate (controlled H gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id (control qubit).
-        q1 : int
-            qubit id (target qubit).
-
-        Returns
-        -------
-        self : instance of QState
-
-        """
-        qstate_operate_qgate(self, kind=cfg.CONTROLLED_H, phase=cfg.DEF_PHASE, qid=[q0, q1])
-        return self
-
-    def cs(self, q0, q1):
-        """
-        operate CS gate (controlled S gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id (control qubit).
-        q1 : int
-            qubit id (target qubit).
-
-        Returns
-        -------
-        self : instance of QState
-
-        """
-        qstate_operate_qgate(self, kind=cfg.CONTROLLED_S, phase=cfg.DEF_PHASE, qid=[q0, q1])
-        return self
-
-    def cs_dg(self, q0, q1):
-        """
-        operate CS dagger gate (controlled S dagger gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id (control qubit).
-        q1 : int
-            qubit id (target qubit).
-
-        Returns
-        -------
-        self : instance of QState
-
-        """
-        qstate_operate_qgate(self, kind=cfg.CONTROLLED_S_, phase=cfg.DEF_PHASE, qid=[q0, q1])
-        return self
-
-    def ct(self, q0, q1):
-        """
-        operate CT gate (controlled T gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id (control qubit).
-        q1 : int
-            qubit id (target qubit).
-
-        Returns
-        -------
-        self : instance of QState
-
-        """
-        qstate_operate_qgate(self, kind=cfg.CONTROLLED_T, phase=cfg.DEF_PHASE, qid=[q0, q1])
-        return self
-
-    def ct_dg(self, q0, q1):
-        """
-        operate CT dagger gate (controlled T dagger gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id (control qubit).
-        q1 : int
-            qubit id (target qubit).
-
-        Returns
-        -------
-        self : instance of QState
-
-        """
-        qstate_operate_qgate(self, kind=cfg.CONTROLLED_T_, phase=cfg.DEF_PHASE, qid=[q0, q1])
-        return self
-
-    def sw(self, q0, q1):
-        """
-        swap gate
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id
-        q1 : int
-            qubit id
-
-        Returns
-        -------
-        self : instance of QState
-
-        """
-        qstate_operate_qgate(self, kind=cfg.SWAP_QUBITS, phase=cfg.DEF_PHASE, qid=[q0, q1])
-        return self
-
-    def cp(self, q0, q1, phase=cfg.DEF_PHASE):
-        """
-        operate CP gate (controlled P gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id (control qubit).
-        q1 : int
-            qubit id (target qubit).
-
-        Returns
-        -------
-        self : instance of QState
-
-        """
-        qstate_operate_qgate(self, kind=cfg.CONTROLLED_P, phase=phase, qid=[q0, q1])
-        return self
-
-    def crx(self, q0, q1, phase=cfg.DEF_PHASE):
-        """
-        operate CRX gate (controlled RX gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id (control qubit).
-        q1 : int
-            qubit id (target qubit).
-        phase : float
-            rotation angle (unit of angle is PI radian).
-
-        Returns
-        -------
-        self : instance of QState
-
-        """
-        qstate_operate_qgate(self, kind=cfg.CONTROLLED_RX, phase=phase, qid=[q0, q1])
-        return self
-
-    def cry(self, q0, q1, phase=cfg.DEF_PHASE):
-        """
-        operate CRY gate (controlled RY gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id (control qubit).
-        q1 : int
-            qubit id (target qubit).
-        phase : float
-            rotation angle (unit of angle is PI radian).
-
-        Returns
-        -------
-        self : instance of QState
-
-        """
-        qstate_operate_qgate(self, kind=cfg.CONTROLLED_RY, phase=phase, qid=[q0, q1])
-        return self
-
-    def crz(self, q0, q1, phase=cfg.DEF_PHASE):
-        """
-        operate CRZ gate (controlled RZ gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id (control qubit).
-        q1 : int
-            qubit id (target qubit).
-        phase : float
-            rotation angle (unit of angle is PI radian).
-
-        Returns
-        -------
-        self : instance of QState
-
-        """
-        qstate_operate_qgate(self, kind=cfg.CONTROLLED_RZ, phase=phase, qid=[q0, q1])
-        return self
-
-    def rxx(self, q0, q1, phase=cfg.DEF_PHASE):
-        """
-        operate Rxx gate.
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id (control qubit).
-        q1 : int
-            qubit id (target qubit).
-        phase : float
-            rotation angle (unit of angle is PI radian).
-
-        Returns
-        -------
-        self : instance of QState
-
-        """
-        qstate_operate_qgate(self, kind=cfg.ROTATION_XX, phase=phase, qid=[q0, q1])
-        return self
-
-    def ryy(self, q0, q1, phase=cfg.DEF_PHASE):
-        """
-        operate Ryy gate.
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id (control qubit).
-        q1 : int
-            qubit id (target qubit).
-        phase : float
-            rotation angle (unit of angle is PI radian).
-
-        Returns
-        -------
-        self : instance of QState
-
-        """
-        qstate_operate_qgate(self, kind=cfg.ROTATION_YY, phase=phase, qid=[q0, q1])
-        return self
-
-    def rzz(self, q0, q1, phase=cfg.DEF_PHASE):
-        """
-        operate Rxx gate.
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id (control qubit).
-        q1 : int
-            qubit id (target qubit).
-        phase : float
-            rotation angle (unit of angle is PI radian).
-
-        Returns
-        -------
-        self : instance of QState
-
-        """
-        qstate_operate_qgate(self, kind=cfg.ROTATION_ZZ, phase=phase, qid=[q0, q1])
-        return self
-
-    # 3-qubit gate
-
-    def ccx(self, q0, q1, q2):
-        """
-        operate CCX gate (toffoli gate, controlled controlled X gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id (control qubit).
-        q1 : int
-            qubit id (control qubit).
-        q2 : int
-            qubit id (target qubit).
-
-        Returns
-        -------
-        self : instance of QState
-
-        """
-        self.cxr(q1, q2).cx(q0, q1).cxr_dg(q1, q2).cx(q0, q1).cxr(q0, q2)
-        return self
-
-    def csw(self, q0, q1, q2):
-        """
-        operate CSW gate (fredkin gate, controlled swap gate).
-
-        Parameters
-        ----------
-        q0 : int
-            qubit id (control qubit).
-        q1 : int
-            qubit id (swap qubit).
-        q2 : int
-            qubit id (swap qubit).
-
-        Returns
-        -------
-        self : instance of QState
-
-        """
-        self.cx(q2, q1).ccx(q0, q1, q2).cx(q2, q1)
-        return self
-
-    # other gate
-
-    def mcx(self, qid=None):
-        """
-        operate MCX gate (multi-controlled X gate).
-
-        Parameters
-        ----------
-        qid : list of int
-            qubit id list [control, control, ... , control, target]
-
-        Returns
-        -------
-        self : instance of QState
-
-        """
-        qstate_mcx(self, qid)
-        return self
-
     # measurement
 
     def measure(self, qid=None):
@@ -1564,63 +864,68 @@ class QState(ctypes.Structure):
         """ bell measurement """
         return qstate_measure_bell_stats(self, qid=qid, shots=shots)
 
-    def operate(self, pp=None, ctrl=None, qctrl=None):
+    # operate gate
+
+    def operate_gate(self, kind=None, qid=None, phase=0.0, **kwargs):
         """
-        operate unitary operator to quantum state.
+        operate gate
 
         Parameters
         ----------
-        pp : instance of PauliProduct
-            pauli product to operate
-        ctrl : int
-            contoroll qubit id for controlled pauli product
-            (this option will be removed near future)
-        qctrl : int
-            contoroll qubit id for controlled pauli product
+        kind : int
+            kind of the gate
+        qid : list
+            quantum id list
+        phase : float
+            phase for rotation gate
 
         Returns
         -------
         self : instance of QState
-            quantum state after operation
+            quantum state
 
         """
-        pauli_list = pp.pauli_list
-        qid = pp.qid
-        factor = pp.factor
-
-        if ctrl is None:
-            ctrl = qctrl
-
-        if ctrl is None:
-            for q, pauli in zip(qid, pauli_list):
-                if pauli == 'X':
-                    self.x(q)
-                elif pauli == 'Y':
-                    self.y(q)
-                elif pauli == 'Z':
-                    self.z(q)
-                else:
-                    continue
+        if kind == cfg.RESET:
+            qstate_reset(self, qid=qid)
+        elif is_unitary_gate(kind):
+            qstate_operate_qgate(self, kind=kind, qid=qid, phase=phase)
         else:
-            if ctrl in qid:
-                raise ValueError("controll and target qubit id conflict")
+            raise ValueError("gate: {} is not supported.".format(cfg.GATE_STRING[kind]))
+        return self
 
-            for q, pauli in zip(qid, pauli_list):
-                if pauli == 'X':
-                    self.cx(ctrl, q)
-                elif pauli == 'Y':
-                    self.cy(ctrl, q)
-                elif pauli == 'Z':
-                    self.cz(ctrl, q)
-                else:
-                    continue
+    # operate quantum circuit
 
-            if factor == -1.+0.j:
-                self.z(ctrl)
-            elif factor == 0.+1.j:
-                self.s(ctrl)
-            elif factor == 0.-1.j:
-                self.s_dg(ctrl)
+    def operate_qcirc(self, qcirc, qctrl=None):
+        """
+        operate quantum circuit
+
+        Parameters
+        ----------
+        qcirc : instance of QCirc
+            quantum circuit
+        qctrl : int
+            control qubit id
+
+        Returns
+        -------
+        self : instance of QState
+            quantum state after executing the quantum circuit
+
+        Notes
+        -----
+        The quantum circut must be unintary.
+
+        """
+        if qcirc.is_unitary() is False:
+            raise ValueError("qcirc must be unitary quantum circuit.")
+        if self.qubit_num < qcirc.qubit_num:
+            raise ValueError("qubit number of quantum state must be equal or larger than the quantum circuit size.")
+
+        if qctrl is None:
+            qstate_operate_qcirc(self, cmem=None, qcirc=qcirc, shots=1, cid=None, out_state=True)
+        else:
+            qc_qctrl = qcirc.add_control(qctrl=qctrl)
+            qstate_operate_qcirc(self, cmem=None, qcirc=qc_qctrl, shots=1, cid=None, out_state=True)
 
         return self
 
@@ -1634,4 +939,5 @@ from qlazy.lib.qstate_c import (qstate_init, qstate_init_with_vector, qstate_res
                                 qstate_inner_product, qstate_get_camp,
                                 qstate_tensor_product, qstate_evolve, qstate_expect_value,
                                 qstate_apply_matrix, qstate_operate_qgate, qstate_measure,
-                                qstate_measure_stats, qstate_measure_bell_stats, qstate_free)
+                                qstate_measure_stats, qstate_measure_bell_stats, qstate_operate_qcirc,
+                                qstate_free)
