@@ -81,7 +81,7 @@ def __convert_to_braket_circuit(qc_qlazy, product):
         if kind is None:
             break
 
-        (kind, qid, para, c, ctrl) = qc_qlazy_tmp.pop_gate()
+        (kind, qid, para, c, ctrl, tag) = qc_qlazy_tmp.pop_gate()
 
         # qid[0]th-qubit has measured and store c-th classical register
         if kind in (cfg.MEASURE, cfg.RESET):
@@ -101,22 +101,26 @@ def __convert_to_braket_circuit(qc_qlazy, product):
         if kind is None:
             break
 
-        (kind, qid, para, c, ctrl) = qc_qlazy_tmp.pop_gate()
+        (kind, qid, para, c, ctrl, tag) = qc_qlazy_tmp.pop_gate()
 
         if kind in (cfg.MEASURE, cfg.RESET):
             continue
 
-        angle = para[0] * np.pi
+        angle = para[0] * para[2] * np.pi
         q0, q1 = qid[0], qid[1]
         __braket_add_qgate(qc_braket, kind, q0, q1, angle, product)
 
     return qc_braket, measured_info
 
-def run(qcirc=None, shots=1, cid=None, backend=None, out_state=False):
+def run(qcirc=None, shots=1, cid=None, backend=None, out_state=False, init=None):
     """ run the quantum circuit on braket_sv """
 
     if qcirc is None:
         raise ValueError("quantum circuit must be specified.")
+    if out_state is True:
+        raise ValueError("output option is not supported.")
+    if init is not None:
+        raise ValueError("init option is not supported.")
 
     qubit_num = qcirc.qubit_num
     cmem_num = qcirc.cmem_num
