@@ -11,11 +11,15 @@ from qiskit.circuit.library.standard_gates import SXdgGate
 import qlazy.config as cfg
 from qlazy.Result import Result
 
-def run(qcirc=None, shots=1, cid=None, backend=None, out_state=False):
+def run(qcirc=None, shots=1, cid=None, backend=None, out_state=False, init=None):
     """ run the quantum circuit """
 
     if qcirc is None:
         raise ValueError("quantum circuit must be specified.")
+    if out_state is True:
+        raise ValueError("output option is not supported.")
+    if init is not None:
+        raise ValueError("init option is not supported.")
 
     qcirc_tmp = qcirc.clone()
 
@@ -40,7 +44,7 @@ def run(qcirc=None, shots=1, cid=None, backend=None, out_state=False):
         if kind is None:
             break
 
-        (kind, qid, para, c, ctrl) = qcirc_tmp.pop_gate()
+        (kind, qid, para, c, ctrl, tag) = qcirc_tmp.pop_gate()
 
         if kind == cfg.MEASURE:
             exist_measurement = True
@@ -52,8 +56,7 @@ def run(qcirc=None, shots=1, cid=None, backend=None, out_state=False):
             qc.reset(qubit_reg[qid[0]])
 
         else:
-            __ibmq_add_qgate(qc, kind, qid, para[0] * np.pi, para[1] * np.pi, para[2] * np.pi,
-                             ctrl, cmem_reg)
+            __ibmq_add_qgate(qc, kind, qid, para[0], para[1], para[2], ctrl, cmem_reg)
 
     # set backend
     if backend.device == 'aer_simulator':
@@ -110,7 +113,9 @@ def run(qcirc=None, shots=1, cid=None, backend=None, out_state=False):
 
     return result
 
-def __ibmq_add_qgate(qc, kind, qid, phase, phase1, phase2, ctrl, cmem_reg):
+def __ibmq_add_qgate(qc, kind, qid, para_phase, para_gphase, para_factor, ctrl, cmem_reg):
+
+    phase = para_phase * para_factor * np.pi
 
     # 1-qubit, 0-parameter gate
     if kind == cfg.PAULI_X:
@@ -204,27 +209,6 @@ def __ibmq_add_qgate(qc, kind, qid, phase, phase1, phase2, ctrl, cmem_reg):
         else:
             qc.p(phase, qid[0]).c_if(cmem_reg[ctrl], 1)
 
-    elif kind == cfg.ROTATION_U1:
-        if ctrl is None:
-            qc.p(phase, qid[0])
-        else:
-            qc.p(phase, qid[0]).c_if(cmem_reg[ctrl], 1)
-
-    # 1-qubit, 2-parameter gate
-    elif kind == cfg.ROTATION_U2:
-        if ctrl is None:
-            qc.u(0.5 * np.pi, phase1, phase, qid[0])
-        else:
-            qc.u(0.5 * np.pi, phase1, phase, qid[0]).c_if(cmem_reg[ctrl], 1)
-
-    # 1-qubit, 3-parameter gate
-    elif kind == cfg.ROTATION_U3:
-        if ctrl is None:
-            qc.u(phase2, phase1, phase, qid[0])
-        else:
-            qc.u(phase2, phase1, phase, qid[0]).c_if(cmem_reg[ctrl], 1)
-
-    # 2-qubit, 0-parameters gate
     elif kind == cfg.CONTROLLED_X:
         if ctrl is None:
             qc.cx(qid[0], qid[1])
@@ -320,25 +304,5 @@ def __ibmq_add_qgate(qc, kind, qid, phase, phase1, phase2, ctrl, cmem_reg):
         else:
             qc.crz(phase, qid[0], qid[1]).c_if(cmem_reg[ctrl], 1)
 
-    elif kind == cfg.CONTROLLED_U1:
-        if ctrl is None:
-            qc.cp(phase, qid[0], qid[1])
-        else:
-            qc.cp(phase, qid[0], qid[1]).c_if(cmem_reg[ctrl], 1)
-
-    # 2-qubit, 2-parameters gate
-    elif kind == cfg.CONTROLLED_U2:
-        if ctrl is None:
-            qc.cu(0.5 * np.pi, phase1, phase, 0, qid[0], qid[1])
-        else:
-            qc.cu(0.5 * np.pi, phase1, phase, 0, qid[0], qid[1]).c_if(cmem_reg[ctrl], 1)
-
-    # 2-qubit, 3-parameters gate
-    elif kind == cfg.CONTROLLED_U3:
-        if ctrl is None:
-            qc.cu(phase2, phase1, phase, 0, qid[0], qid[1])
-        else:
-            qc.cu(phase2, phase1, phase, 0, qid[0], qid[1]).c_if(cmem_reg[ctrl], 1)
-
     else:
         raise ValueError("unknown gate")
