@@ -12,16 +12,18 @@ MAX_QUBIT_NUM = 30
 
 DEF_SHOTS = 1
 
-DEF_TAG = 'DEFAULT'
-
-DEF_PHASE  = 0.0
-DEF_ANGLE  = 0.0
+DEF_PHASE   = 0.0
+DEF_GPHASE  = 0.0
+DEF_FACTOR  = 1.0
+DEF_ANGLE   = 0.0
 
 BELL_PHI_PLUS  = 0
 BELL_PHI_MINUS = 3
 BELL_PSI_PLUS  = 1
 BELL_PSI_MINUS = 2
 
+TAG_STRLEN = 64
+
 # Kind
 
 CIRC           = 1
@@ -116,6 +118,9 @@ GATE_KIND = {
     'cu2': CONTROLLED_U2,
     'cu3': CONTROLLED_U3,
     'sw': SWAP_QUBITS,
+    'rxx': ROTATION_XX,
+    'ryy': ROTATION_XX,
+    'rzz': ROTATION_ZZ,
     'measure': MEASURE,
     'measure_x': MEASURE_X,
     'measure_y': MEASURE_Y,
@@ -160,6 +165,9 @@ GATE_STRING = {
     CONTROLLED_U2:'cu2',
     CONTROLLED_U3:'cu3',
     SWAP_QUBITS:'sw',
+    ROTATION_XX:'rxx',
+    ROTATION_YY:'ryy',
+    ROTATION_ZZ:'rzz',
     MEASURE:'measure',
     MEASURE_X:'measure_x',
     MEASURE_Y:'measure_y',
